@@ -48,12 +48,18 @@ RcppExport SEXP cpbart(
    SEXP _binaryOffset,
    SEXP _itau,
 //   SEXP _iM, // number of shards for Modified LISA MCMC
+   SEXP _idart,
+   SEXP _ia,
+   SEXP _ib,
+   SEXP _irho,
+   SEXP _iaug,
    SEXP _inkeeptrain,
    SEXP _inkeeptest,
-   SEXP _inkeeptestme,
+//   SEXP _inkeeptestme,
    SEXP _inkeeptreedraws,
    SEXP _inprintevery,
-   SEXP _treesaslists
+//   SEXP _treesaslists,
+   SEXP _Xinfo
 )
 {
 
@@ -69,7 +75,9 @@ RcppExport SEXP cpbart(
    Rcpp::NumericVector  xpv(_ixp);
    double *ixp = &xpv[0];
    size_t m = Rcpp::as<int>(_im);
-   size_t nc = Rcpp::as<int>(_inc);
+   //size_t nc = Rcpp::as<int>(_inc);
+   Rcpp::IntegerVector _nc(_inc);
+   int *numcut = &_nc[0];
    size_t nd = Rcpp::as<int>(_ind);
    size_t burn = Rcpp::as<int>(_iburn);
    double mybeta = Rcpp::as<double>(_ipower);
@@ -77,26 +85,51 @@ RcppExport SEXP cpbart(
    double binaryOffset = Rcpp::as<double>(_binaryOffset);
    double tau = Rcpp::as<double>(_itau);
 //   double rootM = sqrt(Rcpp::as<double>(_iM));
-
+   bool dart;
+   if(Rcpp::as<int>(_idart)==1) dart=true;
+   else dart=false;
+   double a = Rcpp::as<double>(_ia);
+   double b = Rcpp::as<double>(_ib);
+   double rho = Rcpp::as<double>(_irho);
+   bool aug;
+   if(Rcpp::as<int>(_iaug)==1) aug=true;
+   else aug=false;
    size_t nkeeptrain = Rcpp::as<int>(_inkeeptrain);
    size_t nkeeptest = Rcpp::as<int>(_inkeeptest);
-   size_t nkeeptestme = Rcpp::as<int>(_inkeeptestme);
+//   size_t nkeeptestme = Rcpp::as<int>(_inkeeptestme);
    size_t nkeeptreedraws = Rcpp::as<int>(_inkeeptreedraws);
    size_t printevery = Rcpp::as<int>(_inprintevery);
-   int treesaslists = Rcpp::as<int>(_treesaslists);
-
-   Rcpp::IntegerMatrix varcount(nkeeptreedraws, p);
+//   int treesaslists = Rcpp::as<int>(_treesaslists);
+   Rcpp::NumericMatrix Xinfo(_Xinfo);
+   //Rcpp::List Xinfo(_Xinfo);
+//   Rcpp::IntegerMatrix varcount(nkeeptreedraws, p);
 
    //return data structures (using Rcpp)
+/*
    Rcpp::NumericVector trmean(n); //train
    Rcpp::NumericVector temean(np);
+*/
    Rcpp::NumericMatrix trdraw(nkeeptrain,n);
    Rcpp::NumericMatrix tedraw(nkeeptest,np);
-   Rcpp::List list_of_lists(nkeeptreedraws*treesaslists);
+//   Rcpp::List list_of_lists(nkeeptreedraws*treesaslists);
+   Rcpp::NumericMatrix varprb(nkeeptreedraws,p);
+   Rcpp::IntegerMatrix varcnt(nkeeptreedraws,p);
 
    //random number generation
    arn gen;
 
+   bart bm(m);
+
+   if(Xinfo.size()>0) {
+     xinfo _xi;
+     _xi.resize(p);
+     for(size_t i=0;i<p;i++) {
+       _xi[i].resize(numcut[i]);
+       //Rcpp::IntegerVector cutpts(Xinfo[i]);
+       for(size_t j=0;j<numcut[i];j++) _xi[i][j]=Xinfo(i, j);
+     }
+     bm.setxinfo(_xi);
+   }
 #else
 
 #define TRDRAW(a, b) trdraw[a][b]
@@ -110,28 +143,35 @@ void cpbart(
    int* iy,		//y, train,  nx1
    double* ixp,		//x, test, pxnp (transposed so rows are contiguous in memory)
    size_t m,		//number of trees
-   size_t nc,		//number of cut points
+   int *numcut, //size_t nc,		//number of cut points
    size_t nd,		//number of kept draws (except for thinnning ..)
    size_t burn,		//number of burn-in draws skipped
    double mybeta,
    double alpha,
    double binaryOffset,
    double tau,
+   bool dart,
+   double a,
+   double b,
+   double rho,
+   bool aug,
    size_t nkeeptrain,
    size_t nkeeptest,
-   size_t nkeeptestme,
+   //size_t nkeeptestme,
    size_t nkeeptreedraws,
    size_t printevery,
-   int treesaslists,
+//   int treesaslists,
    unsigned int n1, // additional parameters needed to call from C++
    unsigned int n2,
+/*
    double* trmean,
    double* temean,
+*/
    double* _trdraw,
    double* _tedraw
 )
 {
-  
+
    //return data structures (using C++)
    std::vector<double*> trdraw(nkeeptrain);
    std::vector<double*> tedraw(nkeeptest);
@@ -139,29 +179,41 @@ void cpbart(
    for(size_t i=0; i<nkeeptrain; ++i) trdraw[i]=&_trdraw[i*n];
    for(size_t i=0; i<nkeeptest; ++i) tedraw[i]=&_tedraw[i*np];
 
+   std::vector< std::vector<size_t> > varcnt;
+   std::vector< std::vector<double> > varprb;
+
    //random number generation
    arn gen(n1, n2);
+
+   bart bm(m);
 #endif
-   
+
+/*
    for(size_t i=0; i<n; ++i) trmean[i]=0.0;
    for(size_t i=0; i<np; ++i) temean[i]=0.0;
+*/
 
    double* iz = new double[n];
 
-   std::stringstream treess;  //string stream to write trees to  
+   std::stringstream treess;  //string stream to write trees to
    treess.precision(10);
    treess << nkeeptreedraws << " " << m << " " << p << endl;
+   // dart iterations
+   std::vector<double> ivarprb (p,0.);
+   std::vector<size_t> ivarcnt (p,0);
 
    printf("*****Into main of pbart\n");
 
-
-   size_t skiptr,skipte,skipteme,skiptreedraws;
+   size_t skiptr,skipte,skiptreedraws;
+   //size_t skiptr,skipte,skipteme,skiptreedraws;
    if(nkeeptrain) {skiptr=nd/nkeeptrain;}
    else skiptr = nd+1;
    if(nkeeptest) {skipte=nd/nkeeptest;}
    else skipte=nd+1;
+/*
    if(nkeeptestme) {skipteme=nd/nkeeptestme;}
    else skipteme=nd+1;
+*/
    if(nkeeptreedraws) {skiptreedraws = nd/nkeeptreedraws;}
    else skiptreedraws=nd+1;
 
@@ -173,21 +225,28 @@ void cpbart(
    printf("x1,x[n*p]: %lf, %lf\n",ix[0],ix[n*p-1]);
    if(np) printf("xp1,xp[np*p]: %lf, %lf\n",ixp[0],ixp[np*p-1]);
    printf("*****Number of Trees: %zu\n",m);
-   printf("*****Number of Cut Points: %zu\n",nc);
+   printf("*****Number of Cut Points: %d ... %d\n", numcut[0], numcut[p-1]);
    printf("*****burn and ndpost: %zu, %zu\n",burn,nd);
-   printf("*****Prior:\nbeta,alpha,tau: %lf,%lf,%lf\n",
+   printf("*****Prior:beta,alpha,tau: %lf,%lf,%lf\n",
                    mybeta,alpha,tau);
    printf("*****binaryOffset: %lf\n",binaryOffset);
+   cout << "*****Dirichlet:sparse,a,b,rho,augment: " << dart << ',' << a << ',' 
+	<< b << ',' << rho << ',' << aug << endl;
+   printf("*****nkeeptrain,nkeeptest,nkeeptreedraws: %zu,%zu,%zu\n",
+               nkeeptrain,nkeeptest,nkeeptreedraws);
+/*
    printf("*****nkeeptrain,nkeeptest,nkeeptestme,nkeeptreedraws: %zu,%zu,%zu,%zu\n",
                nkeeptrain,nkeeptest,nkeeptestme,nkeeptreedraws);
+*/
    printf("*****printevery: %zu\n",printevery);
-   printf("*****skiptr,skipte,skipteme,skiptreedraws: %zu,%zu,%zu,%zu\n",skiptr,skipte,skipteme,skiptreedraws);
+   //printf("*****skiptr,skipte,skipteme,skiptreedraws: %zu,%zu,%zu,%zu\n",skiptr,skipte,skipteme,skiptreedraws);
+   printf("*****skiptr,skipte,skiptreedraws: %zu,%zu,%zu\n",skiptr,skipte,skiptreedraws);
 
    //--------------------------------------------------
-   bart bm(m);
+   //bart bm(m);
    bm.setprior(alpha,mybeta,tau);
-   bm.setdata(p,n,ix,iz,nc);
-
+   bm.setdata(p,n,ix,iz,numcut);
+   bm.setdart(a,b,rho,aug,dart);
    //--------------------------------------------------
 //init
   for(size_t k=0; k<n; k++) {
@@ -200,7 +259,7 @@ void cpbart(
    //--------------------------------------------------
    //temporary storage
    //out of sample fit
-   double* fhattest=0; //posterior mean for prediction
+   double* fhattest=0; 
    if(np) { fhattest = new double[np]; }
 
    //--------------------------------------------------
@@ -209,9 +268,9 @@ void cpbart(
    //size_t index;
    size_t trcnt=0; //count kept train draws
    size_t tecnt=0; //count kept test draws
-   size_t temecnt=0; //count test draws into posterior mean
-   size_t treedrawscnt=0; //count kept bart draws
-   bool keeptest,keeptestme,keeptreedraw;
+//   size_t temecnt=0; //count test draws into posterior mean
+//   size_t treedrawscnt=0; //count kept bart draws
+   bool keeptest,/*keeptestme,*/keeptreedraw;
 
    time_t tp;
    int time1 = time(&tp);
@@ -219,10 +278,11 @@ void cpbart(
 
    for(size_t i=0;i<(nd+burn);i++) {
       if(i%printevery==0) printf("done %zu (out of %lu)\n",i,nd+burn);
+      if(i==(burn/2)&&dart) bm.startdart();
       //draw bart
       bm.draw(1., gen);
       //bm.draw(rootM, gen);
- 
+
   for(size_t k=0; k<n; k++) {
     if(iy[k]==0) iz[k]= -rtnorm(-bm.f(k), binaryOffset, 1., gen);
     else iz[k]=rtnorm(bm.f(k), -binaryOffset, 1., gen);
@@ -230,7 +290,7 @@ void cpbart(
 
       if(i>=burn) {
 
-         for(size_t k=0;k<n;k++) trmean[k]+=bm.f(k);
+         //for(size_t k=0;k<n;k++) trmean[k]+=bm.f(k);
          if(nkeeptrain && (((i-burn+1) % skiptr) ==0)) {
             //index = trcnt*n;;
             //for(size_t k=0;k<n;k++) trdraw[index+k]=bm.f(k);
@@ -238,61 +298,82 @@ void cpbart(
             trcnt+=1;
          }
          keeptest = nkeeptest && (((i-burn+1) % skipte) ==0) && np;
-         keeptestme = nkeeptestme && (((i-burn+1) % skipteme) ==0) && np;
-         if(keeptest || keeptestme) bm.predict(p,np,ixp,fhattest);
+         //keeptestme = nkeeptestme && (((i-burn+1) % skipteme) ==0) && np;
+         //if(keeptest || keeptestme) bm.predict(p,np,ixp,fhattest);
          if(keeptest) {
+	   bm.predict(p,np,ixp,fhattest);
             //index=tecnt*np;
             //for(size_t k=0;k<np;k++) tedraw[index+k]=fhattest[k];
             for(size_t k=0;k<np;k++) TEDRAW(tecnt,k)=fhattest[k];
             tecnt+=1;
          }
+/*
          if(keeptestme) {
             for(size_t k=0;k<np;k++) temean[k]+=fhattest[k];
             temecnt+=1;
          }
+*/
          keeptreedraw = nkeeptreedraws && (((i-burn+1) % skiptreedraws) ==0);
          if(keeptreedraw) {
-	   #ifndef NoRcpp
-	   Rcpp::List lists(m*treesaslists);
-	   #endif
+//	   #ifndef NoRcpp
+//	   Rcpp::List lists(m*treesaslists);
+//	   #endif
 
             for(size_t j=0;j<m;j++) {
 	      treess << bm.gettree(j);
-
+/*
 	      #ifndef NoRcpp
 	      varcount.row(treedrawscnt)=varcount.row(treedrawscnt)+bm.gettree(j).tree2count(p);
 	      if(treesaslists) lists(j)=bm.gettree(j).tree2list(xi, 0., 1.);
 	      #endif
+*/
 	    }
 	    #ifndef NoRcpp
-	    if(treesaslists) list_of_lists(treedrawscnt)=lists;
+//	    if(treesaslists) list_of_lists(treedrawscnt)=lists;
+	    ivarcnt=bm.getnv();
+	    ivarprb=bm.getpv();
+	    size_t k=(i-burn)/skiptreedraws;
+	    for(size_t j=0;j<p;j++){
+	      varcnt(k,j)=ivarcnt[j];
+	      //varcnt(i-burn,j)=ivarcnt[j];
+	      varprb(k,j)=ivarprb[j];
+	      //varprb(i-burn,j)=ivarprb[j];
+	    }
+            #else
+	    varcnt.push_back(bm.getnv());
+	    varprb.push_back(bm.getpv());
 	    #endif
 
-            treedrawscnt +=1;
+            //treedrawscnt +=1;
          }
       }
    }
    int time2 = time(&tp);
    printf("time: %ds\n",time2-time1);
+/*
    for(size_t k=0;k<n;k++) trmean[k]/=nd;
    for(size_t k=0;k<np;k++) temean[k]/=temecnt;
+*/
    printf("check counts\n");
-   printf("trcnt,tecnt,temecnt,treedrawscnt: %zu,%zu,%zu,%zu\n",trcnt,tecnt,temecnt,treedrawscnt);
+   printf("trcnt,tecnt: %zu,%zu\n",trcnt,tecnt);
+   //printf("trcnt,tecnt,temecnt,treedrawscnt: %zu,%zu,%zu,%zu\n",trcnt,tecnt,temecnt,treedrawscnt);
    //--------------------------------------------------
    //PutRNGstate();
 
-   if(fhattest) delete[] fhattest;   
+   if(fhattest) delete[] fhattest;
    delete[] iz;
 
 #ifndef NoRcpp
    //--------------------------------------------------
    //return
    Rcpp::List ret;
-   ret["yhat.train.mean"]=trmean;
+   //ret["yhat.train.mean"]=trmean;
    ret["yhat.train"]=trdraw;
-   ret["yhat.test.mean"]=temean;
+   //ret["yhat.test.mean"]=temean;
    ret["yhat.test"]=tedraw;
-   ret["varcount"]=varcount;
+   //ret["varcount"]=varcount;
+   ret["varcount"]=varcnt;
+   ret["varprob"]=varprb;
 
    //for(size_t i=0;i<m;i++) {
     //  bm.gettree(i).pr();
@@ -311,7 +392,7 @@ void cpbart(
    //treesL["numx"] = Rcpp::wrap<int>(p); //in cutpoints
    treesL["cutpoints"] = xiret;
    treesL["trees"]=Rcpp::CharacterVector(treess.str());
-   if(treesaslists) treesL["lists"]=list_of_lists;
+   //if(treesaslists) treesL["lists"]=list_of_lists;
    ret["treedraws"] = treesL;
 
    return ret;
