@@ -273,7 +273,7 @@ void cwbart(
 
    for(size_t i=0;i<(nd+burn);i++) {
       if(i%printevery==0) printf("done %zu (out of %lu)\n",i,nd+burn);
-      if(i==floor(burn/2)&&dart) bm.startdart();
+      if(i==(burn/2)&&dart) bm.startdart();
       //draw bart
       bm.draw(svec,gen);
       //draw sigma
